#include <WiFi.h>

int outputPin = 2; // Digital pin to output square wave
int controlPin = 5; // Digital pin to indicate sweep active
int inverseControlPin = 4; // Digital pin to indicate sweep inactive
int currentFreq = 200; // Start frequency
int maxFreq = 1600;
unsigned long lastUpdate = 0; // Timestamp for frequency update
unsigned long lastFSRRead = 0; // Timestamp for FSR reading
const int sweepInterval = 2; // Frequency change interval in milliseconds

// Wi-Fi credentials
const char* ssid = "DavidPhone"; // Replace with your Wi-Fi SSID
const char* password = "dpchotspot"; // Replace with your Wi-Fi password

WiFiServer server(80); // Start a TCP server on port 80
WiFiClient client; // Handle client connection

bool sweepActive = false; // Control whether the sweep is active

void setup() {
  Serial.begin(115200);
  pinMode(outputPin, OUTPUT);
  pinMode(controlPin, OUTPUT);
  pinMode(inverseControlPin, OUTPUT);
  digitalWrite(controlPin, LOW); // Ensure controlPin is LOW initially
  digitalWrite(inverseControlPin, HIGH); // Ensure inverseControlPin is HIGH initially

  // Connect to Wi-Fi
  Serial.print("Connecting to Wi-Fi");
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nWi-Fi connected.");
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());

  // Start the TCP server
  server.begin();
  Serial.println("Server started. Waiting for clients...");
}

void loop() {
  // Check for client connection
  if (!client || !client.connected()) {
    client = server.available(); // Wait for a client to connect
    if (client) {
      Serial.println("Client connected.");
    }
  }

  // Check for start command from client
  if (!sweepActive && client && client.connected() && client.available()) {
    char command = client.read();
    if (command == 's') {
      sweepActive = true;
      currentFreq = 200; // Reset to start frequency
      digitalWrite(controlPin, HIGH); // Indicate that the sweep is active
      digitalWrite(inverseControlPin, LOW); // Indicate that the sweep is inactive (opposite state)
      client.println("Starting fast frequency sweep.");
    }
  }

  // Continuously sweep frequencies if the sweep is active
  if (sweepActive && currentFreq <= maxFreq) {
    // Update frequency at defined intervals
    if (millis() - lastUpdate >= sweepInterval) {
      lastUpdate = millis();

      // Send current frequency to the client
      if (client && client.connected()) {
        client.print("Frequency: ");
        client.println(currentFreq);
      }

      // Move to the next frequency
      currentFreq++;
    }

    // Generate PWM signal at the current frequency
    generatePWM(outputPin, currentFreq, 128); // 50% duty cycle
  } else if (sweepActive && currentFreq > maxFreq) {
    digitalWrite(outputPin, LOW); // Stop PWM output
    digitalWrite(controlPin, LOW); // Indicate that the sweep has ended
    digitalWrite(inverseControlPin, HIGH); // Indicate sweep is inactive (opposite state)
    if (client && client.connected()) {
      client.println("Sweep completed.");
    }
    currentFreq = 200; // Reset to start frequency
    sweepActive = false; // Stop the sweep until 's' is sent again
  }
}

// Non-blocking PWM function
void generatePWM(int pin, int frequency, int dutyCycle) {
  static unsigned long lastToggleTime = 0; // Timestamp of the last toggle
  static bool pinState = LOW; // Current state of the pin (LOW or HIGH)

  // Calculate period and high/low times
  int period = 1000000L / frequency; // Period in microseconds
  int highTime = period * dutyCycle / 255; // High time based on duty cycle
  int lowTime = period - highTime; // Low time

  unsigned long currentTime = micros(); // Current time in microseconds

  // Determine if it's time to toggle the pin state
  if ((pinState == HIGH && currentTime - lastToggleTime >= highTime) ||
      (pinState == LOW && currentTime - lastToggleTime >= lowTime)) {
    pinState = !pinState; // Toggle the pin state
    digitalWrite(pin, pinState); // Update the pin state
    lastToggleTime = currentTime; // Reset the toggle timer
  }
}
