#include <WiFi.h>

const char* ssid = "DavidPhone"; // Replace with your Wi-Fi SSID
const char* password = "dpchotspot"; // Replace with your Wi-Fi password

WiFiServer server(80); // Wi-Fi server on port 80

unsigned long startMillis = 0;
unsigned long recordingDuration = 0;

void setup() {
  Serial.begin(115200); // High baud rate for serial monitor
  analogReadResolution(12); // Set ADC resolution to 12 bits

  Serial.println("Connecting to Wi-Fi...");
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("\nWi-Fi connected!");
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());

  server.begin(); // Start the server
}

void loop() {
  WiFiClient client = server.available(); // Check for a new client

  if (client) {
    Serial.println("Client connected!");
    client.println("Enter duration in seconds to start recording:");

    while (client.connected()) {
      if (client.available() > 0) {
        String input = client.readStringUntil('\n');
        input.trim();

        if (input.toInt() > 0) {
          recordingDuration = input.toInt() * 1000; // Convert seconds to milliseconds
          startMillis = millis();
          client.println("Starting data recording...");
          Serial.println("Recording started for " + String(recordingDuration / 1000) + " seconds.");

          // Prepare buffer for batch data transmission
          const int bufferSize = 512; // Define buffer size
          char buffer[bufferSize];
          int bufferIndex = 0;

          // Record and batch send data
          while (millis() - startMillis < recordingDuration) {
            unsigned long currentTime = micros(); // Timestamp in microseconds
            int sensorValue = analogRead(A2);    // Read analog value

            // Format data and add to buffer
            bufferIndex += snprintf(buffer + bufferIndex, sizeof(buffer) - bufferIndex, "%lu,%d\n", currentTime, sensorValue);

            // Check if buffer is nearly full
            if (bufferIndex >= bufferSize - 50) {
              client.write(buffer, bufferIndex); // Send buffer to client
              bufferIndex = 0;                  // Reset buffer index
            }
          }

          // Send any remaining data in buffer
          if (bufferIndex > 0) {
            client.write(buffer, bufferIndex);
          }

          client.println("Recording complete.");
          Serial.println("Recording complete.");
          recordingDuration = 0; // Reset recording duration
        }
      }
    }

    client.stop(); // Close connection
    Serial.println("Client disconnected.");
  }
}
